# -*- coding: utf-8 -*-
from typing import Optional


class InternalError(Exception):
    """Indica un errore interno non ricondicibile ad altri sistemi."""


class ValidationError(Exception):
    pass


class AAMSException(Exception):
    error_code: int
    description: Optional[str] = None

    def __init__(self, error_code, description: Optional[str] = None):
        self.error_code = error_code
        self.description = description
        super().__init__(self.description or f"AAMSException {self.error_code}")

    def __str__(self):
        return "{} ({}): {}".format(self.__class__.__name__, self.error_code, self.description)

    def __repr__(self):
        return self.__str__()

"""
Di seguito l’elenco, non esaustivo, degli errori che possono essere restituiti dal sistema
centrale in risposta ai messaggi inviati dal sistema di elaborazione del concessionario.
L’elenco aggiornato è sempre disponibile nella sezione “giochi di abilità” dell’ area
riservata del sito di AAMS.
"""


ALL_ERRORS = "__all__"
GENERIC_ERROR = ([ALL_ERRORS], "Errore non gestito")
HEADER = "Header"

AAMS_EXCEPTIONS = {
    1000: ([ALL_ERRORS], "Errore generico lettura messaggio"),
    1001: ([ALL_ERRORS], "Elaborazione in corso - attendere prego"),
    1003: ([ALL_ERRORS], "Codice transazione si riferisce a differente richiesta gia effettuata"),
    1004: ([ALL_ERRORS], "Versione protocollo errato o mancante"),
    1010: ([ALL_ERRORS], "Firma non verificata"),
    1020: ([280, 340], "Codice concessionario trasmittente inesistente o non abilitato"),
    1040: (ALL_ERRORS, "Identificativo sessione errato o mancante"),
    1050: ([HEADER], "Codice concessionario trasmittente non abilitatoad inviare il messaggio"),
    1060: ([ALL_ERRORS], "Lunghezza messaggio errata"),
    1070: ([ALL_ERRORS], "Messaggio gia trasmesso"),
    1080: ([ALL_ERRORS], "Codice messaggio errato"),
    1090: ([HEADER, ALL_ERRORS], "Data/Ora errata"),
    1091: ([HEADER, 200, 220, 280, 310, 320, 340, 400, 580, 780], "Data/Ora indicata successiva alla attuale"),
    1092: ([400, 600], "Data successiva alla data di inizio Validita del servizio"),
    1100: ([200, 240], "Percentuale montepremi errata"),
    1120: ([230, 260, 320, 330], "Codice biglietto errato o inesistente"),
    1140: ([240], "Importi piano dei premi non congruenti con montepremi"),
    1150: ([280, 430, 630], "Importo accredito non congruente con importo premi"),
    1160: ([HEADER], "Codice concessionario proponente inesistente o non abilitato"),
    1190: ([HEADER, 200, 830, 831], "Codice gioco errato o mancante"),
    1191: ([HEADER], "Gioco non abilitato per il concessionario ed FSC"),
    1200: ([200, 400, 600], "Identificativo della sessione di gioco non univoco"),
    1210: ([830, 831], "Codice piattaforma non valido"),
    1220: ([200, 220, 420, 430, 510, 620, 630], "Importo nominale diritto di partecipazione <=0 o superiore al massimo consentito"),
    1221: ([220, 280, 420, 430, 580, 620, 630, 780], "Importo bonus <0 o superiore al consentito"),
    1222: ([430, 630], "Importo nominale jackpot <0"),
    1223: ([430, 630], "Importo nominale jackpot <0"),
    1224: ([430, 630], "Importo nominale Puntato <0"),
    1225: ([430, 630], "Importo nominale Vinto <0"),
    1226: ([430, 630], "Importo inizio e fine partecipazione, somma puntate e restituito per vincita incongruenti"),
    1230: ([200], "PRM valore attributo (percentuale) errato"),
    1240: ([200], "SMG valore errato o mancante"),
    1241: ([200], "SMG valore errato o mancante"),
    1250: ([200], "MNG valore inferiore a 1"),
    1260: ([200], "MXG valore inferiore a 2"),
    1261: ([200], "MXG valore inferiore a MNG"),
    1270: ([200, 220, 250, 300, 320, 340, 400, 420, 620], "Data precedente ad inizio sessione"),
    1271: ([220, 250, 300, 420, 620], "Data successiva alla fine sessione"),
    1272: ([300], "Data successiva alla convalida sessione"),
    1273: ([300], "Data precedente alla convalida sessione"),
    1280: ([590, 790], "Quadratura in corso, inviare il messaggio successivamente"),
    1281: ([200, 220], "Flag riacquisto mancante o errato"),
    1290: ([220, 420, 510, 620], "Pseudonimo giocatore assente"),
    # TODO
    1300: ([], ""),
    1310: ([], ""),
    1320: ([], ""),
    1330: ([], ""),
    1333: ([], ""),
    1335: ([], ""),
    1340: ([], ""),
    1350: ([], ""),
    1351: ([], ""),
    1352: ([], ""),
    1353: ([], ""),
    1354: ([], ""),
    1355: ([], ""),
    1360: ([], ""),
    1370: ([], ""),
    1380: ([], ""),
    1381: ([], ""),
    1390: ([], ""),

    1400: ([HEADER], "Codice Concessionario proponente mancante"),
    1410: ([HEADER], "Codice Concessionario trasmittente mancante"),
    1420: ([HEADER], "CodiceTransazione mancante"),
    1421: ([260, 330, 340, 420, 430, 620, 630], "Identificativo di Partecipazione mancante o errato"),
    1422: ([420, 620], "Progressivo di Partecipazione mancante o errato"),
    1423: ([420, 620], "Progressivo partecipazione o Identificativo partecipazione non correttamente valorizzato."),
    1424: ([260], "Importo totale vincita da bonus mancante o errato"),
    1425: ([420, 430, 620, 630], "Il giocatore risulta aver chiuso la sua partecipazione al tavolo"),
    1426: ([420, 430, 620, 630], "Il giocatore risulta aver superato il limite dell'importo giocabile"),
    1427: ([420, 430, 620, 630], "Il giocatore non si e mai seduto al tavolo oppure è errato il prog di partecipazione"),
    1428: ([420, 430, 620, 630], "Il giocatore è gia seduto al tavolo"),
    1429: ([260], "Importo parziale da bonus errato o mancante"),
    1430: ([HEADER], "Data assente"),
    1431: ([500, 700], "Ci sono ancora giocatori seduti al tavolo"),
    1432: ([420, 430, 620, 630], "Importo di partecipazione inferiore al valore minimo dichiarato per il tavolo"),
    1433: ([420, 430, 620, 630], "Importo di partecipazione superiore al valore massimo dichiarato per il tavolo"),
    1440: ([HEADER, 280, 310, 320], "Tipo messaggio assente"),
    1441: ([ALL_ERRORS], "Tipo messaggio non previsto per modalita di trasmissone del gioco"),
    1442: ([ALL_ERRORS], "Gioco incongruente con modalita di trasmissione del gioco"),
    1443: ([ALL_ERRORS], "Tipo messaggio non previsto per tipo gioco"),
    1450: ([HEADER], "Lunghezza body assente"),
    1460: ([200, 220], "Orario assente"),
    1470: ([200, 400, 600], "BON - valore bonus < > B o F"),
    1471: ([200], "JCK - valore jackpot mancante o errato"),
    1472: ([200], "CUP - valore attributo CUP mancante o errato"),
    1473: ([220, 420], "Importo bonus errato o mancante"),
    1474: ([430, 620, "630"], "Codice rete errato o mancante"),
    1475: ([260], "Importo parziale da Jackpot mancante o errato"),
    1476: ([260], "Importi parziali errati"),
    1477: ([400, 600], "Valore attributo MNI negativo"),
    1478: ([400, 600], "Valore attributo MXI negativo"),
    1479: ([200, 400], "Importo jackpot aggiuntivo mancante o errato"),
    1480: ([400, 420], "Importo Partecipazione da bonus errato o mancante"),
    1481: ([400, 600], "Valore attributo MXI inferiore al valore dell'attributo MNI"),
    1483: ([620], "Il tavolo ha gia raggiunto il numero max di partecipanti"),
    1484: ([600], "valore attributo RAK negativo"),
    1485: ([600], "valore attributo RAK errato"),
    1486: ([600], "valore attributo TAV errato"),
    # todo
    1487: ([], ""),
    1488: ([], ""),
    1489: ([], ""),
    1490: ([], ""),
    1491: ([], ""),
    1492: ([], ""),
    1493: ([], ""),
    1494: ([], ""),
    1495: ([], ""),
    1496: ([], ""),
    1497: ([], ""),
    1500: ([], ""),
    1510: ([], ""),
    1520: ([], ""),
    1530: ([], ""),
    1540: ([], ""),
    1550: ([], ""),

    1560: ([HEADER], "Concessionario proponente temporaneamente disabilitato"),
    1570: ([240], "Numero premi non congruente con quello dichiarato"),
    1571: ([260], "Numero vincitori non congruente con quello dichiarato"),
    1580: ([300], "Numero vincitori dichiarato diverso da quello effettivo"),
    1590: ([240], "Percentuale montepremi dichiarato nel msg200 diverso da quello dichiarato nel msg240"),
    1600: ([240], "Percentuale montepremi errata"),
    1603: ([400, 600], "Numero attributi inferiore a quelli obbligatori"),
    1610: ([220, 420, 510, 620], "Indirizzo IP assente"),
    1620: ([220, 420, 510, 620], "Regione errata o mancante"),
    1630: ([220, 420, 510, 620], "Lunghezza pseudonimo del giocatore errata o mancante"),
    1631: ([220, 420, 500, 510, 620], "Pseudonimo del giocatore associato ad altro conto di gioco"),
    1640: ([420, 620], "Lunghezza pseudonimo del giocatore nel circuito errata o mancante"),
    1650: ([220, 420, 510, 620], "Lunghezza pseudonimo del giocatore incongruente con valore pseudonimo"),
    1660: ([420, 620], "Lunghezza pseudonimo del giocatore nel circuito incongruente con valore pseudonimo"),
    1670: ([240], "Somma vincite non equivalente al montepremi erogato"),
    1671: ([240], "Somma jackpot non congruente con jackpot erogato"),
    # todo
    1672: ([], ""),
    1700: ([], ""),
    1710: ([], ""),
    1720: ([], ""),
    1740: ([], ""),
    1750: ([], ""),
    1760: ([], ""),
    1770: ([], ""),
    1781: ([], ""),
    1810: ([], ""),
    1820: ([], ""),
    1821: ([], ""),
    1822: ([], ""),
    1830: ([], ""),
    1870: ([], ""),
    1871: ([], ""),
    1872: ([], ""),
    1880: ([], ""),
    1881: ([], ""),
    1890: ([], ""),
    1910: ([], ""),
    1920: ([], ""),
    1921: ([], ""),
    1930: ([], ""),
    1940: ([], ""),
    1950: ([], ""),
    1960: ([], ""),
    1970: ([], ""),
    1990: ([], ""),
    2000: ([], ""),
    2010: ([], ""),
    2011: ([], ""),
    2020: ([], ""),
    2030: ([], ""),
    2031: ([], ""),
    2040: ([], ""),
    2050: ([], ""),
    2060: ([], ""),
    2061: ([], ""),
    2070: ([], ""),
    2071: ([], ""),
    2073: ([], ""),
    2074: ([], ""),
    2075: ([], ""),
    2076: ([], ""),
    2077: ([], ""),
    2078: ([], ""),
    2079: ([], ""),
    2080: ([], ""),
    2081: ([], ""),
    2090: ([], ""),
    2100: ([], ""),
    2110: ([], ""),
    2111: ([], ""),
    2112: ([], ""),
    2130: ([], ""),

    2140: ([510, 580], "Importo nominale da play bonus <0 o superiore al consentito"),
    2150: ([240], "Importo premio non inferiore a premi gia trasmessi"),
    2151: ([240], "Importo vincita non inferiore a vincite già trasmesse"),
    2160: ([240], "Numero premio da inserire iniziale mancante o errato"),
    2170: ([240], "Numero premio da inserire finale mancante o errato"),
    2171: ([240], "Montepremi parziale mancante o errato"),
    2172: ([240], "Importo jackpot parziale mancante o errato"),
    2173: ([240], "Numero premi presenti nell'invio mancante o errato"),
    2175: ([240], "Flag rettifica mancante o errato"),
    2180: ([240], "Importi dei premi trasmesso non congruenti con montepremi parziale"),
    2190: ([240], "Importo montepremi parziale trasmesso congruente col montepremi erogato"),
    2191: ([240], "Importo jackpot parziale trasmesso non congruente col jackpot erogato"),
    2192: ([240], "Importo jackpot parziale aggiuntivo trasmesso non congruente col jackpot erogato"),
    2200: ([ALL_ERRORS], "Giorno mancante o errato"),
    2210: ([ALL_ERRORS], "Mese mancante o errato"),
    2220: ([ALL_ERRORS], "Anno mancante o errato"),
    2230: ([ALL_ERRORS], "Ora mancante o errata"),
    2240: ([ALL_ERRORS], "Minuti mancanti o errati"),
    # TODO DA 1250 A 2483





    2484: ([200], "2484 - NMG valore bonus errato"),
    2485: ([370, 570, 770], "Codice tipo sessione errato o mancante"),
    2486: ([375, 575, 775], "Il giocatore non risulta seduto al tavolo"),
    2487: ([840], "Certificato non valido"),
    2488: ([840], "Certificato scaduto"),
    2489: ([840], "CA del certificato non valida"),
    2490: ([840], "Campo num_seriale_cert mancante o errato"),
    2491: ([840], "Campo lun_cert mancante o errato"),
    2492: ([250], "convalida"),
    2493: ([220, 420, 510, 620], "Campo cod_tipo_tag mancante o errato"),
    2494: ([831], "Campo prog_versione_cert mancante o errato"),
    2495: ([831], "Campo prog_sub_versione_cert mancante o errato"),
    2496: ([831], "errato"),
    2497: ([831], "Nome_modulo_critico non comunicato da EVA"),
    2498: ([831], "comunicato da EVA"),

    9000: ([ALL_ERRORS], "Errore operazioni di aggiornamento banche dati del sistema di convalida")
}
